@@ -1,6 +1,6 @@
 /*
  * sar/sadc: Report system activity
- * (C) 1999-2013 by Sebastien Godard (sysstat <at> orange.fr)
+ * (C) 1999-2014 by Sebastien Godard (sysstat <at> orange.fr)
  */
 
 #ifndef _SA_H
@@ -242,11 +242,14 @@
  */
 #define AO_SELECTED		0x02
 /*
- * Indicate that, when registered again, activity counters will get back
- * the values they had when they were unregistered (eg. CPUs, which can
- * be disabled/enabled on the fly).
+ * When appending data to a file, the number of items (for every activity)
+ * is forced to that of the file (number of network interfaces, serial lines,
+ * etc.) Exceptions are volatile activities (like A_CPU) whose number of items
+ * is related to the number of CPUs: If current machine has a different number
+ * of CPU than that of the file (but is equal to sa_last_cpu_nr) then data
+ * will be appended with a number of items equal to that of the machine.
  */
-#define AO_REMANENT		0x04
+#define AO_VOLATILE		0x04
 /*
  * Indicate that the interval of time, given to f_print() function
  * displaying statistics, should be the interval of time in jiffies
@@ -268,7 +271,7 @@
 
 #define IS_COLLECTED(m)		(((m) & AO_COLLECTED)        == AO_COLLECTED)
 #define IS_SELECTED(m)		(((m) & AO_SELECTED)         == AO_SELECTED)
-#define IS_REMANENT(m)		(((m) & AO_REMANENT)         == AO_REMANENT)
+#define IS_VOLATILE(m)		(((m) & AO_VOLATILE)         == AO_VOLATILE)
 #define NEED_GLOBAL_ITV(m)	(((m) & AO_GLOBAL_ITV)       == AO_GLOBAL_ITV)
 #define CLOSE_MARKUP(m)		(((m) & AO_CLOSE_MARKUP)     == AO_CLOSE_MARKUP)
 #define HAS_MULTIPLE_OUTPUTS(m)	(((m) & AO_MULTIPLE_OUTPUTS) == AO_MULTIPLE_OUTPUTS)
@@ -451,7 +454,7 @@ struct activity {
  * 	|
  * 	|--                         --|
  * 	|                             |
- * 	| file_activity structure     | * sa_nr_act
+ * 	| file_activity structure     | * sa_act_nr
  * 	|                             |
  * 	|--                         --|
  * 	|                             |
@@ -464,8 +467,9 @@ struct activity {
  * 	|--                         --|
  *
  * (*)Note: If it's a special record, we may find a comment instead of
- * statistics (R_COMMENT record type) or even nothing at all (R_RESTART
- * record type).
+ * statistics (R_COMMENT record type) or, if it's a R_RESTART record type,
+ * <sa_nr_vol_act> structures (of type file_activity) for the volatile
+ * activities.
  ***************************************************************************
  */
 
@@ -480,7 +484,7 @@ struct activity {
  * Modified to indicate that the format of the file is
  * no longer compatible with that of previous sysstat versions.
  */
-#define FORMAT_MAGIC	0x2171
+#define FORMAT_MAGIC	0x2173
 
 /* Structure for file magic header data */
 struct file_magic {
@@ -499,6 +503,14 @@ struct file_magic {
 	unsigned char  sysstat_patchlevel;
 	unsigned char  sysstat_sublevel;
 	unsigned char  sysstat_extraversion;
+	/*
+	 * Size of file's header (size of file_header structure used by file).
+	 */
+	unsigned int header_size;
+	/*
+	 * Padding. Reserved for future use while avoiding a format change.
+	 */
+	unsigned char pad[64];
 };
 
 #define FILE_MAGIC_SIZE	(sizeof(struct file_magic))
@@ -511,9 +523,18 @@ struct file_header {
 	 */
 	unsigned long sa_ust_time	__attribute__ ((aligned (8)));
 	/*
-	 * Number of activities saved in the file
+	 * Number of CPU items (1 .. CPU_NR + 1) for the last sample in file.
 	 */
-	unsigned int sa_nr_act		__attribute__ ((aligned (8)));
+	unsigned int sa_last_cpu_nr	__attribute__ ((aligned (8)));
+	/*
+	 * Number of activities saved in file.
+	 */
+	unsigned int sa_act_nr;
+	/*
+	 * Number of volatile activities in file. This is the number of
+	 * file_activity structures saved after each restart mark in file.
+	 */
+	unsigned int sa_vol_act_nr;
 	/*
 	 * Current day, month and year.
 	 * No need to save DST (Daylight Saving Time) flag, since it is not taken
@@ -840,6 +861,11 @@ extern void
 	print_report_hdr(unsigned int, struct tm *, struct file_header *, int);
 extern void
 	read_file_stat_bunch(struct activity * [], int, int, int, struct file_activity *);
+extern __nr_t
+	read_vol_act_structures(int, struct activity * [], char *, struct file_magic *,
+			        unsigned int);
+extern int
+	reallocate_vol_act_structures(struct activity * [], unsigned int, unsigned int);
 extern int
 	sa_fread(int, void *, int, int);
 extern void
